import { world, system, Vector, BlockPermutation, ItemStack } from "@minecraft/server";

// Hello user! Not sure why you are in here but hello :3
// Want a coding challange? Try adding some sort of lockpicking system on iron doors!

function changeDoorState(block, state = -1) {
	const isTop = block.permutation.getState("upper_block_bit");
	const topBlock = isTop ? block : block.dimension.getBlock(Vector.add(block.location, Vector.up));
	const isHinged = topBlock.permutation.getState("door_hinge_bit");
	const bottomBlock = isTop ? block.dimension.getBlock(Vector.add(block.location, Vector.down)) : block;
	const direction = bottomBlock.permutation.getState("direction");
	const permutation = BlockPermutation.resolve(block.typeId);
	const newState = !bottomBlock.permutation.getState("open_bit");
	bottomBlock.setPermutation(permutation.withState("open_bit", state === -1 ? newState : state).withState("direction", direction));
	topBlock.setPermutation(permutation.withState("upper_block_bit", true).withState("door_hinge_bit", isHinged));
	return {
		state: newState,
		isHinged: isHinged,
		direction: direction
	};
}

world.beforeEvents.playerInteractWithBlock.subscribe(function (event) {
	const typeId = event.block.typeId;
	if (!typeId.includes("door") || typeId.includes("trap") || typeId === "minecraft:iron_door") return;
	event.cancel = true;
	system.run(function () {
		if (event.player.isSneaking) {
			const result = changeDoorState(event.block);
			for (const player of event.player.dimension.getPlayers({ location: event.player.location, maxDistance: 8 })) {
				player.playSound(result.state ? "random.door_open" : "random.door_close", { location: event.block.location, volume: 0.4 });
			}
			return;
		}
		const result = changeDoorState(event.block);
		const adjacentBlock = event.player.dimension.getBlock(Vector.add(event.block.location, Vector.multiply([Vector.forward, Vector.left, Vector.back, Vector.right][result.direction], result.isHinged ? -1 : 1)));
		if (adjacentBlock.typeId.includes("door") && !adjacentBlock.typeId.includes("trap") && adjacentBlock.typeId !== "minecraft:iron_door") changeDoorState(adjacentBlock, result.state);
		for (const player of event.player.dimension.getPlayers({ location: event.player.location, maxDistance: 10 })) {
			player.playSound(result.state ? "random.door_open" : "random.door_close", { location: event.block.location });
		}
	})
});